#include <GL/glut.h>
#include <stdio.h>
#include <map>
#include "Triangle.h"
#pragma warning(disable:4786)


void Mesh::Draw()
{
	vector<Triangle>::iterator i;
	glEnable(GL_LIGHTING);
	glBegin(GL_TRIANGLES);
	for(i=T.begin();i!=T.end();++i)
	{
		if(has_color) glColor3fv((*i).c[0].v);
		glNormal((*i).n[0]);
		glVertex((*i).v[0]);
		if(has_color) glColor3fv((*i).c[1].v);
		glNormal((*i).n[1]);
		glVertex((*i).v[1]);
		if(has_color) glColor3fv((*i).c[2].v);
		glNormal((*i).n[2]);
		glVertex((*i).v[2]);
	}
	glEnd();
}

void Mesh::Open(char *filename)
{
	if(strcmp(filename+strlen(filename)-4,"rawc")==0 ||
		 strcmp(filename+strlen(filename)-4,"RAWC")==0 ){
		 OpenColor(filename);
		 return ;
	}
	FILE *fp;
	fp=fopen(filename,"r");
	if(fp==0)	{
		printf("Unable to open %s",filename);
		exit(-1);
	}

	has_color=false;
	Triangle tt;
	T.clear();
	float xf,yf,zf;
	int nc;
	while(1)
	{
		nc=fscanf(fp,"%f %f %f",&xf,&yf,&zf);
		if(nc!=3) break;
		tt.v[0]=Point3f(xf,yf,zf);
		nc=fscanf(fp,"%f %f %f",&xf,&yf,&zf);
		if(nc!=3) break;
		tt.v[1]=Point3f(xf,yf,zf);
		nc=fscanf(fp,"%f %f %f",&xf,&yf,&zf);
		if(nc!=3) break;
		tt.v[2]=Point3f(xf,yf,zf);
		tt.CalcNormal();
		T.push_back(tt);
	}
	fclose(fp);
	printf("Read %i triangles\n",T.size());
}

void Mesh::OpenColor(char *filename)
{
	FILE *fp;
	fp=fopen(filename,"r");
	if(fp==0)		{
			printf("Unable to open %s",filename);
			exit(-1);
	}
  has_color=true;
	Triangle tt;
	T.clear();
	float xf,yf,zf;
	float cr,cg,cb;
	int nc;
	while(1)
	{
		nc= fscanf(fp,"%f %f %f",&xf,&yf,&zf);
		tt.v[0]=Point3f(xf,yf,zf);
		nc+= fscanf(fp,"%f %f %f",&xf,&yf,&zf);
		tt.v[1]=Point3f(xf,yf,zf);
		nc+= fscanf(fp,"%f %f %f",&xf,&yf,&zf);
		tt.v[2]=Point3f(xf,yf,zf);
		nc+=fscanf(fp,"%f %f %f",&cr,&cg,&cb);
		tt.c[0]=Point3f(cr,cg,cb);
		nc+=fscanf(fp,"%f %f %f",&cr,&cg,&cb);
		tt.c[1]=Point3f(cr,cg,cb);
		nc+=fscanf(fp,"%f %f %f",&cr,&cg,&cb);
		tt.c[2]=Point3f(cr,cg,cb);
		tt.CalcNormal();
		if(nc!=18) break;
		T.push_back(tt);
	}
	fclose(fp);
	printf("Read %i triangles\n",T.size());

}

void Mesh::Smooth()
{
	map<Point3f,Point3f> NN;  // the vertex to normal map
	vector<Triangle>::iterator i;
	// clear and create all the map entry
	for(i=T.begin();i!=T.end();++i){ 
			NN[(*i).v[0]]=Point3f(0,0,0);
			NN[(*i).v[1]]=Point3f(0,0,0);
			NN[(*i).v[2]]=Point3f(0,0,0);
		}
	// accumulate normals per vertex 
	for(i=T.begin();i!=T.end();++i){
		NN[(*i).v[0]]+=(*i).n[0];
		NN[(*i).v[1]]+=(*i).n[1];
		NN[(*i).v[2]]+=(*i).n[2];
		}

	// Normalize all the normals stored in the map
	map<Point3f,Point3f>::iterator in;
	for(in=NN.begin();in!=NN.end();++in)
			(*in).second.Normalize();
	
	// Distribute results into triangles
	for(i=T.begin();i!=T.end();++i){
			(*i).n[0]=NN[(*i).v[0]];
			(*i).n[1]=NN[(*i).v[1]];
			(*i).n[2]=NN[(*i).v[2]];
		}
}

void Triangle::CalcNormal()
{
	n[0]=(v[1]-v[0])^(v[2]-v[0]);
	n[0].Normalize();
	n[1]=n[0];
	n[2]=n[0];
}

void Mesh::Normalize(float size)
{
	Point3f min=T[0].v[0],max=T[0].v[0];

	int j;
	vector<Triangle>::iterator i;
	for(i=T.begin();i!=T.end();++i){
		for(j=0;j<3;++j)
		{
		  if(min.v[0] > (*i).v[j].v[0]) min.v[0] = (*i).v[j].v[0];
			if(min.v[1] > (*i).v[j].v[1]) min.v[1] = (*i).v[j].v[1];
			if(min.v[2] > (*i).v[j].v[2]) min.v[2] = (*i).v[j].v[2];
	
		  if(max.v[0] < (*i).v[j].v[0]) max.v[0] = (*i).v[j].v[0];
	    if(max.v[1] < (*i).v[j].v[1]) max.v[1] = (*i).v[j].v[1];
			if(max.v[2] < (*i).v[j].v[2]) max.v[2] = (*i).v[j].v[2];
		}
	}

	Point3f d=max-min;
	float maxd=d.x();
	if(maxd<d.y()) maxd=d.y();
	if(maxd<d.z()) maxd=d.z();
	Point3f c=(min+max)/2;
	for(i=T.begin();i!=T.end();++i)
		for(j=0;j<3;++j)
		{
				(*i).v[j]-=c;
				(*i).v[j]/=maxd;
		}
}
